@@ -3,12 +3,11 @@
  * - RR-type-specific code, and the machinery to call it
  */
 /*
- *  This file is
- *    Copyright (C) 1997-1999 Ian Jackson <ian@davenant.greenend.org.uk>
- *
- *  It is part of adns, which is
- *    Copyright (C) 1997-2000 Ian Jackson <ian@davenant.greenend.org.uk>
- *    Copyright (C) 1999-2000 Tony Finch <dot@dotat.at>
+ *  This file is part of adns, which is
+ *    Copyright (C) 1997-2000,2003,2006  Ian Jackson
+ *    Copyright (C) 1999-2000,2003,2006  Tony Finch
+ *    Copyright (C) 1991 Massachusetts Institute of Technology
+ *  (See the file INSTALL for full details.)
  *  
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -34,8 +33,10 @@
 
 #include "internal.h"
 
-#define R_NOMEM           return adns_s_nomemory
-#define CSP_ADDSTR(s)     do { if (!adns__vbuf_appendstr(vb,(s))) R_NOMEM; } while (0)
+#define R_NOMEM       return adns_s_nomemory
+#define CSP_ADDSTR(s) do {			\
+    if (!adns__vbuf_appendstr(vb,(s))) R_NOMEM;	\
+  } while (0)
 
 /*
  * order of sections:
@@ -61,9 +62,13 @@
  * _mailbox                   (pap +pap_mailbox822)
  * _rp                        (pa)
  * _soa                       (pa,mf,cs)
+ * _srv*                      (qdpl,(pap),pa,mf,di,(csp),cs,postsort)
+ * _byteblock                 (mf)
+ * _opaque                    (pa,cs)
  * _flat                      (mf)
  *
  * within each section:
+ *    qdpl_*
  *    pap_*
  *    pa_*
  *    dip_*
@@ -72,6 +77,7 @@
  *    mf_*
  *    csp_*
  *    cs_*
+ *    postsort_*
  */
 
 /*
@@ -172,7 +178,8 @@ static void mf_manyistr(adns_query qu, void *datap) {
  * _txt   (pa,cs)
  */
 
-static adns_status pa_txt(const parseinfo *pai, int cbyte, int max, void *datap) {
+static adns_status pa_txt(const parseinfo *pai, int cbyte,
+			  int max, void *datap) {
   adns_rr_intstr **rrp= datap, *table, *te;
   const byte *dgram= pai->dgram;
   int ti, tc, l, startbyte;
@@ -235,7 +242,8 @@ static adns_status cs_hinfo(vbuf *vb, const void *datap) {
  * _inaddr   (pa,dip,di)
  */
 
-static adns_status pa_inaddr(const parseinfo *pai, int cbyte, int max, void *datap) {
+static adns_status pa_inaddr(const parseinfo *pai, int cbyte,
+			     int max, void *datap) {
   struct in_addr *storeto= datap;
   
   if (max-cbyte != 4) return adns_s_invaliddata;
@@ -248,7 +256,8 @@ static int search_sortlist(adns_state ads, struct in_addr ad) {
   int i;
   
   for (i=0, slp=ads->sortlist;
-       i<ads->nsortlist && !((ad.s_addr & slp->mask.s_addr) == slp->base.s_addr);
+       i<ads->nsortlist &&
+	 !((ad.s_addr & slp->mask.s_addr) == slp->base.s_addr);
        i++, slp++);
   return i;
 }
@@ -263,7 +272,8 @@ static int dip_inaddr(adns_state ads, struct in_addr a, struct in_addr b) {
   return bi<ai;
 }
 
-static int di_inaddr(adns_state ads, const void *datap_a, const void *datap_b) {
+static int di_inaddr(adns_state ads,
+		     const void *datap_a, const void *datap_b) {
   const struct in_addr *ap= datap_a, *bp= datap_b;
 
   return dip_inaddr(ads,*ap,*bp);
@@ -282,7 +292,8 @@ static adns_status cs_inaddr(vbuf *vb, const void *datap) {
  * _addr   (pa,di,csp,cs)
  */
 
-static adns_status pa_addr(const parseinfo *pai, int cbyte, int max, void *datap) {
+static adns_status pa_addr(const parseinfo *pai, int cbyte,
+			   int max, void *datap) {
   adns_rr_addr *storeto= datap;
   const byte *dgram= pai->dgram;
 
@@ -367,7 +378,8 @@ static adns_status cs_domain(vbuf *vb, const void *datap) {
   return csp_domain(vb,*domainp);
 }
 
-static adns_status pa_dom_raw(const parseinfo *pai, int cbyte, int max, void *datap) {
+static adns_status pa_dom_raw(const parseinfo *pai, int cbyte,
+			      int max, void *datap) {
   char **rrp= datap;
   adns_status st;
 
@@ -382,7 +394,8 @@ static adns_status pa_dom_raw(const parseinfo *pai, int cbyte, int max, void *da
  * _host_raw   (pa)
  */
 
-static adns_status pa_host_raw(const parseinfo *pai, int cbyte, int max, void *datap) {
+static adns_status pa_host_raw(const parseinfo *pai, int cbyte,
+			       int max, void *datap) {
   char **rrp= datap;
   adns_status st;
 
@@ -406,7 +419,8 @@ static adns_status pap_findaddrs(const parseinfo *pai, adns_rr_hostaddr *ha,
   adns_status st;
   
   for (rri=0, naddrs=-1; rri<count; rri++) {
-    st= adns__findrr_anychk(pai->qu, pai->serv, pai->dgram, pai->dglen, cbyte_io,
+    st= adns__findrr_anychk(pai->qu, pai->serv, pai->dgram,
+			    pai->dglen, cbyte_io,
 			    &type, &class, &ttl, &rdlen, &rdstart,
 			    pai->dgram, pai->dglen, dmstart, &ownermatched);
     if (st) return st;
@@ -416,7 +430,8 @@ static adns_status pap_findaddrs(const parseinfo *pai, adns_rr_hostaddr *ha,
     if (naddrs == -1) {
       naddrs= 0;
     }
-    if (!adns__vbuf_ensure(&pai->qu->vb, (naddrs+1)*sizeof(adns_rr_addr))) R_NOMEM;
+    if (!adns__vbuf_ensure(&pai->qu->vb, (naddrs+1)*sizeof(adns_rr_addr)))
+      R_NOMEM;
     adns__update_expires(pai->qu,ttl,pai->now);
     st= pa_addr(pai, rdstart,rdstart+rdlen,
 		pai->qu->vb.buf + naddrs*sizeof(adns_rr_addr));
@@ -446,7 +461,8 @@ static void icb_hostaddr(adns_query parent, adns_query child) {
   rrp->astatus= st;
   rrp->naddrs= (st>0 && st<=adns_s_max_tempfail) ? -1 : cans->nrrs;
   rrp->addrs= cans->rrs.addr;
-  adns__transfer_interim(child, parent, rrp->addrs, rrp->naddrs*sizeof(adns_rr_addr));
+  adns__transfer_interim(child, parent, rrp->addrs,
+			 rrp->naddrs*sizeof(adns_rr_addr));
 
   if (parent->children.head) {
     LIST_LINK_TAIL(ads->childw,parent);
@@ -506,7 +522,8 @@ static adns_status pap_hostaddr(const parseinfo *pai, int *cbyte_io,
   return adns_s_ok;
 }
 
-static adns_status pa_hostaddr(const parseinfo *pai, int cbyte, int max, void *datap) {
+static adns_status pa_hostaddr(const parseinfo *pai, int cbyte,
+			       int max, void *datap) {
   adns_rr_hostaddr *rrp= datap;
   adns_status st;
 
@@ -517,7 +534,8 @@ static adns_status pa_hostaddr(const parseinfo *pai, int cbyte, int max, void *d
   return adns_s_ok;
 }
 
-static int dip_hostaddr(adns_state ads, const adns_rr_hostaddr *ap, const adns_rr_hostaddr *bp) {
+static int dip_hostaddr(adns_state ads,
+			const adns_rr_hostaddr *ap, const adns_rr_hostaddr *bp) {
   if (ap->astatus != bp->astatus) return ap->astatus;
   if (ap->astatus) return 0;
 
@@ -528,7 +546,8 @@ static int dip_hostaddr(adns_state ads, const adns_rr_hostaddr *ap, const adns_r
 		    bp->addrs[0].addr.inet.sin_addr);
 }
 
-static int di_hostaddr(adns_state ads, const void *datap_a, const void *datap_b) {
+static int di_hostaddr(adns_state ads,
+		       const void *datap_a, const void *datap_b) {
   const adns_rr_hostaddr *ap= datap_a, *bp= datap_b;
 
   return dip_hostaddr(ads, ap,bp);
@@ -592,7 +611,8 @@ static adns_status cs_hostaddr(vbuf *vb, const void *datap) {
  * _mx_raw   (pa,di)
  */
 
-static adns_status pa_mx_raw(const parseinfo *pai, int cbyte, int max, void *datap) {
+static adns_status pa_mx_raw(const parseinfo *pai, int cbyte,
+			     int max, void *datap) {
   const byte *dgram= pai->dgram;
   adns_rr_intstr *rrp= datap;
   adns_status st;
@@ -621,7 +641,8 @@ static int di_mx_raw(adns_state ads, const void *datap_a, const void *datap_b) {
  * _mx   (pa,di)
  */
 
-static adns_status pa_mx(const parseinfo *pai, int cbyte, int max, void *datap) {
+static adns_status pa_mx(const parseinfo *pai, int cbyte,
+			 int max, void *datap) {
   const byte *dgram= pai->dgram;
   adns_rr_inthostaddr *rrp= datap;
   adns_status st;
@@ -713,7 +734,8 @@ static void icb_ptr(adns_query parent, adns_query child) {
   adns__query_fail(parent,adns_s_inconsistent);
 }
 
-static adns_status pa_ptr(const parseinfo *pai, int dmstart, int max, void *datap) {
+static adns_status pa_ptr(const parseinfo *pai, int dmstart,
+			  int max, void *datap) {
   static const char *(expectdomain[])= { DNS_INADDR_ARPA };
   
   char **rrp= datap;
@@ -741,15 +763,18 @@ static adns_status pa_ptr(const parseinfo *pai, int dmstart, int max, void *data
     for (i=0; i<4; i++) {
       st= adns__findlabel_next(&fls,&lablen,&labstart); assert(!st);
       if (lablen<=0 || lablen>3) return adns_s_querydomainwrong;
-      memcpy(labbuf, pai->qu->query_dgram + labstart, lablen);  labbuf[lablen]= 0;
-      ipv[3-i]= strtoul(labbuf,&ep,10);  if (*ep) return adns_s_querydomainwrong;
+      memcpy(labbuf, pai->qu->query_dgram + labstart, lablen);
+      labbuf[lablen]= 0;
+      ipv[3-i]= strtoul(labbuf,&ep,10);
+      if (*ep) return adns_s_querydomainwrong;
       if (lablen>1 && pai->qu->query_dgram[labstart]=='0')
 	return adns_s_querydomainwrong;
     }
     for (i=0; i<sizeof(expectdomain)/sizeof(*expectdomain); i++) {
       st= adns__findlabel_next(&fls,&lablen,&labstart); assert(!st);
       l= strlen(expectdomain[i]);
-      if (lablen != l || memcmp(pai->qu->query_dgram + labstart, expectdomain[i], l))
+      if (lablen != l ||
+	  memcmp(pai->qu->query_dgram + labstart, expectdomain[i], l))
 	return adns_s_querydomainwrong;
     }
     st= adns__findlabel_next(&fls,&lablen,0); assert(!st);
@@ -806,7 +831,8 @@ static void mf_intstrpair(adns_query qu, void *datap) {
  * _hinfo   (pa)
  */
 
-static adns_status pa_hinfo(const parseinfo *pai, int cbyte, int max, void *datap) {
+static adns_status pa_hinfo(const parseinfo *pai, int cbyte,
+			    int max, void *datap) {
   adns_rr_intstrpair *rrp= datap;
   adns_status st;
   int i;
@@ -825,8 +851,8 @@ static adns_status pa_hinfo(const parseinfo *pai, int cbyte, int max, void *data
  * _mailbox   (pap,cs)
  */
 
-static adns_status pap_mailbox822(const parseinfo *pai, int *cbyte_io, int max,
-				  char **mb_r) {
+static adns_status pap_mailbox822(const parseinfo *pai,
+				  int *cbyte_io, int max, char **mb_r) {
   int lablen, labstart, i, needquote, c, r, neednorm;
   const unsigned char *p;
   char *str;
@@ -883,7 +909,7 @@ static adns_status pap_mailbox822(const parseinfo *pai, int *cbyte_io, int max,
 
 static adns_status pap_mailbox(const parseinfo *pai, int *cbyte_io, int max,
 			       char **mb_r) {
-  if (pai->qu->typei->type & adns__qtf_mail822) {
+  if (pai->qu->typei->typekey & adns__qtf_mail822) {
     return pap_mailbox822(pai, cbyte_io, max, mb_r);
   } else {
     return pap_domain(pai, cbyte_io, max, mb_r, pdf_quoteok);
@@ -898,7 +924,8 @@ static adns_status csp_mailbox(vbuf *vb, const char *mailbox) {
  * _rp   (pa,cs)
  */
 
-static adns_status pa_rp(const parseinfo *pai, int cbyte, int max, void *datap) {
+static adns_status pa_rp(const parseinfo *pai, int cbyte,
+			 int max, void *datap) {
   adns_rr_strpair *rrp= datap;
   adns_status st;
 
@@ -927,7 +954,8 @@ static adns_status cs_rp(vbuf *vb, const void *datap) {
  * _soa   (pa,mf,cs)
  */
 
-static adns_status pa_soa(const parseinfo *pai, int cbyte, int max, void *datap) {
+static adns_status pa_soa(const parseinfo *pai, int cbyte,
+			  int max, void *datap) {
   adns_rr_soa *rrp= datap;
   const byte *dgram= pai->dgram;
   adns_status st;
@@ -976,6 +1004,235 @@ static adns_status cs_soa(vbuf *vb, const void *datap) {
   return adns_s_ok;
 }
 
+/*
+ * _srv*  (pa*2,di,cs*2,qdpl,postsort)
+ */
+
+static adns_status qdpl_srv(adns_state ads,
+			    const char **p_io, const char *pe, int labelnum,
+			    char label_r[DNS_MAXDOMAIN], int *ll_io,
+			    adns_queryflags flags,
+			    const typeinfo *typei) {
+  int useflags;
+  const char *p_orig;
+  adns_status st;
+
+  if (labelnum < 2 && !(flags & adns_qf_quoteok_query)) {
+    useflags= adns_qf_quoteok_query;
+    p_orig= *p_io;
+  } else {
+    useflags= flags;
+    p_orig= 0;
+  }
+  st= adns__qdpl_normal(ads, p_io,pe, labelnum,label_r, ll_io, useflags,typei);
+  if (st) return st;
+
+  if (p_orig) {
+    int ll= *ll_io;
+    if (!ll || label_r[0]!='_')
+      return adns_s_querydomaininvalid;
+    if (memchr(p_orig+1, '\\', pe - (p_orig+1)))
+      return adns_s_querydomaininvalid;
+  }
+  return adns_s_ok;
+}
+
+static adns_status pap_srv_begin(const parseinfo *pai, int *cbyte_io, int max,
+				 adns_rr_srvha *rrp
+				   /* might be adns_rr_srvraw* */) {
+  const byte *dgram= pai->dgram;
+  int ti, cbyte;
+
+  cbyte= *cbyte_io;
+  if ((*cbyte_io += 6) > max) return adns_s_invaliddata;
+  
+  rrp->priority= GET_W(cbyte, ti);
+  rrp->weight=   GET_W(cbyte, ti);
+  rrp->port=     GET_W(cbyte, ti);
+  return adns_s_ok;
+}
+
+static adns_status pa_srvraw(const parseinfo *pai, int cbyte,
+			     int max, void *datap) {
+  adns_rr_srvraw *rrp= datap;
+  adns_status st;
+
+  st= pap_srv_begin(pai,&cbyte,max,datap);
+  if (st) return st;
+  
+  st= pap_domain(pai, &cbyte, max, &rrp->host,
+		 pai->qu->flags & adns_qf_quoteok_anshost ? pdf_quoteok : 0);
+  if (st) return st;
+  
+  if (cbyte != max) return adns_s_invaliddata;
+  return adns_s_ok;
+}
+
+static adns_status pa_srvha(const parseinfo *pai, int cbyte,
+			    int max, void *datap) {
+  adns_rr_srvha *rrp= datap;
+  adns_status st;
+
+  st= pap_srv_begin(pai,&cbyte,max,datap);       if (st) return st;
+  st= pap_hostaddr(pai, &cbyte, max, &rrp->ha);  if (st) return st;
+  if (cbyte != max) return adns_s_invaliddata;
+  return adns_s_ok;
+}
+
+static void mf_srvraw(adns_query qu, void *datap) {
+  adns_rr_srvraw *rrp= datap;
+  adns__makefinal_str(qu, &rrp->host);
+}
+
+static void mf_srvha(adns_query qu, void *datap) {
+  adns_rr_srvha *rrp= datap;
+  mfp_hostaddr(qu,&rrp->ha);
+}
+
+static int di_srv(adns_state ads, const void *datap_a, const void *datap_b) {
+  const adns_rr_srvraw *ap= datap_a, *bp= datap_b;
+    /* might be const adns_rr_svhostaddr* */
+
+  if (ap->priority < bp->priority) return 0;
+  if (ap->priority > bp->priority) return 1;
+  return 0;
+}
+
+static adns_status csp_srv_begin(vbuf *vb, const adns_rr_srvha *rrp
+				   /* might be adns_rr_srvraw* */) {
+  char buf[30];
+  sprintf(buf,"%u %u %u ", rrp->priority, rrp->weight, rrp->port);
+  CSP_ADDSTR(buf);
+  return adns_s_ok;
+}
+
+static adns_status cs_srvraw(vbuf *vb, const void *datap) {
+  const adns_rr_srvraw *rrp= datap;
+  adns_status st;
+  
+  st= csp_srv_begin(vb,(const void*)rrp);  if (st) return st;
+  return csp_domain(vb,rrp->host);
+}
+
+static adns_status cs_srvha(vbuf *vb, const void *datap) {
+  const adns_rr_srvha *rrp= datap;
+  adns_status st;
+
+  st= csp_srv_begin(vb,(const void*)datap);  if (st) return st;
+  return csp_hostaddr(vb,&rrp->ha);
+}
+
+static void postsort_srv(adns_state ads, void *array, int nrrs,
+			 const struct typeinfo *typei) {
+  /* we treat everything in the array as if it were an adns_rr_srvha
+   * even though the array might be of adns_rr_srvraw.  That's OK
+   * because they have the same prefix, which is all we access.
+   * We use typei->rrsz, too, rather than naive array indexing, of course.
+   */
+  char *workbegin, *workend, *search, *arrayend;
+  const adns_rr_srvha *rr;
+  union { adns_rr_srvha ha; adns_rr_srvraw raw; } rrtmp;
+  int cpriority, totalweight, runtotal;
+  long randval;
+
+  for (workbegin= array, arrayend= workbegin + typei->rrsz * nrrs;
+       workbegin < arrayend;
+       workbegin= workend) {
+    cpriority= (rr=(void*)workbegin)->priority;
+    
+    for (workend= workbegin, totalweight= 0;
+	 workend < arrayend && (rr=(void*)workend)->priority == cpriority;
+	 workend += typei->rrsz) {
+      totalweight += rr->weight;
+    }
+
+    /* Now workbegin..(workend-1) incl. are exactly all of the RRs of
+     * cpriority.  From now on, workbegin points to the `remaining'
+     * records: we select one record at a time (RFC2782 `Usage rules'
+     * and `Format of the SRV RR' subsection `Weight') to place at
+     * workbegin (swapping with the one that was there, and then
+     * advance workbegin. */
+    for (;
+	 workbegin + typei->rrsz < workend; /* don't bother if just one */
+	 workbegin += typei->rrsz) {
+      
+      randval= nrand48(ads->rand48xsubi);
+      randval %= (totalweight + 1);
+        /* makes it into 0..totalweight inclusive; with 2^10 RRs,
+	 * totalweight must be <= 2^26 so probability nonuniformity is
+	 * no worse than 1 in 2^(31-26) ie 1 in 2^5, ie
+	 *  abs(log(P_intended(RR_i) / P_actual(RR_i)) <= log(2^-5).
+	 */
+
+      for (search=workbegin, runtotal=0;
+	   (runtotal += (rr=(void*)search)->weight) < randval;
+	   search += typei->rrsz);
+      assert(search < arrayend);
+      totalweight -= rr->weight;
+      if (search != workbegin) {
+	memcpy(&rrtmp, workbegin, typei->rrsz);
+	memcpy(workbegin, search, typei->rrsz);
+	memcpy(search, &rrtmp, typei->rrsz);
+      }
+    }
+  }
+  /* tests:
+   *  dig -t srv _srv._tcp.test.iwj.relativity.greenend.org.uk.
+   *   ./adnshost_s -t srv- _sip._udp.voip.net.cam.ac.uk.
+   *   ./adnshost_s -t srv- _jabber._tcp.jabber.org
+   */
+}
+
+/*
+ * _byteblock   (mf)
+ */
+
+static void mf_byteblock(adns_query qu, void *datap) {
+  adns_rr_byteblock *rrp= datap;
+  void *bytes= rrp->data;
+  adns__makefinal_block(qu,&bytes,rrp->len);
+  rrp->data= bytes;
+}
+
+/*
+ * _opaque   (pa,cs)
+ */
+
+static adns_status pa_opaque(const parseinfo *pai, int cbyte,
+			     int max, void *datap) {
+  adns_rr_byteblock *rrp= datap;
+
+  rrp->len= max - cbyte;
+  rrp->data= adns__alloc_interim(pai->qu, rrp->len);
+  if (!rrp->data) R_NOMEM;
+  memcpy(rrp->data, pai->dgram + cbyte, rrp->len);
+  return adns_s_ok;
+}
+
+static adns_status cs_opaque(vbuf *vb, const void *datap) {
+  const adns_rr_byteblock *rrp= datap;
+  char buf[10];
+  int l;
+  unsigned char *p;
+
+  sprintf(buf,"\\# %d",rrp->len);
+  CSP_ADDSTR(buf);
+  
+  for (l= rrp->len, p= rrp->data;
+       l>=4;
+       l -= 4, p += 4) {
+    sprintf(buf," %02x%02x%02x%02x",p[0],p[1],p[2],p[3]);
+    CSP_ADDSTR(buf);
+  }
+  for (;
+       l>0;
+       l--, p++) {
+    sprintf(buf," %02x",*p);
+    CSP_ADDSTR(buf);
+  }
+  return adns_s_ok;
+}
+  
 /*
  * _flat   (mf)
  */
@@ -991,43 +1248,57 @@ static void mf_flat(adns_query qu, void *data) { }
 #define DEEP_MEMB(memb) TYPESZ_M(memb), mf_##memb, cs_##memb
 #define FLAT_MEMB(memb) TYPESZ_M(memb), mf_flat, cs_##memb
 
-#define DEEP_TYPE(code,rrt,fmt,memb,parser,comparer,printer) \
- { adns_r_##code, rrt, fmt, TYPESZ_M(memb), mf_##memb, printer, parser, comparer }
-#define FLAT_TYPE(code,rrt,fmt,memb,parser,comparer,printer) \
- { adns_r_##code, rrt, fmt, TYPESZ_M(memb), mf_flat, printer, parser, comparer }
+#define DEEP_TYPE(code,rrt,fmt,memb,parser,comparer,printer)	\
+ { adns_r_##code, rrt,fmt,TYPESZ_M(memb), mf_##memb,		\
+      printer,parser,comparer, adns__qdpl_normal,0 }
+#define FLAT_TYPE(code,rrt,fmt,memb,parser,comparer,printer)	\
+ { adns_r_##code, rrt,fmt,TYPESZ_M(memb), mf_flat,		\
+     printer,parser,comparer, adns__qdpl_normal,0 }
+#define XTRA_TYPE(code,rrt,fmt,memb,parser,comparer,printer,qdpl,postsort) \
+ { adns_r_##code, rrt,fmt,TYPESZ_M(memb), mf_##memb,			   \
+    printer,parser,comparer,qdpl,postsort }
 
 static const typeinfo typeinfos[] = {
 /* Must be in ascending order of rrtype ! */
-/* mem-mgmt code  rrt     fmt     member      parser      comparer    printer       */
-  		    		     		 	     		 	       
-FLAT_TYPE(a,      "A",     0,     inaddr,     pa_inaddr,  di_inaddr,  cs_inaddr     ),
-DEEP_TYPE(ns_raw, "NS",   "raw",  str,        pa_host_raw,0,          cs_domain     ),
-DEEP_TYPE(cname,  "CNAME", 0,     str,        pa_dom_raw, 0,          cs_domain     ),
-DEEP_TYPE(soa_raw,"SOA",  "raw",  soa,        pa_soa,     0,          cs_soa        ),
-DEEP_TYPE(ptr_raw,"PTR",  "raw",  str,        pa_host_raw,0,          cs_domain     ),
-DEEP_TYPE(hinfo,  "HINFO", 0,     intstrpair, pa_hinfo,   0,          cs_hinfo      ),
-DEEP_TYPE(mx_raw, "MX",   "raw",  intstr,     pa_mx_raw,  di_mx_raw,  cs_inthost    ),
-DEEP_TYPE(txt,    "TXT",   0,     manyistr,   pa_txt,     0,          cs_txt        ),
-DEEP_TYPE(rp_raw, "RP",   "raw",  strpair,    pa_rp,      0,          cs_rp         ),
- 		     	                                     		   	       
-FLAT_TYPE(addr,   "A",  "addr",   addr,       pa_addr,    di_addr,    cs_addr       ),
-DEEP_TYPE(ns,     "NS", "+addr",  hostaddr,   pa_hostaddr,di_hostaddr,cs_hostaddr   ),
-DEEP_TYPE(ptr,    "PTR","checked",str,        pa_ptr,     0,          cs_domain     ),
-DEEP_TYPE(mx,     "MX", "+addr",  inthostaddr,pa_mx,      di_mx,      cs_inthostaddr),
- 		     	                                     		      
-DEEP_TYPE(soa,    "SOA","822",    soa,        pa_soa,     0,          cs_soa        ),
-DEEP_TYPE(rp,     "RP", "822",    strpair,    pa_rp,      0,          cs_rp         ),
+/* mem-mgmt code  rrt     fmt   member   parser      comparer  printer */
+
+FLAT_TYPE(a,      "A",     0,   inaddr,  pa_inaddr,  di_inaddr,cs_inaddr     ),
+DEEP_TYPE(ns_raw, "NS",   "raw",str,     pa_host_raw,0,        cs_domain     ),
+DEEP_TYPE(cname,  "CNAME", 0,   str,     pa_dom_raw, 0,        cs_domain     ),
+DEEP_TYPE(soa_raw,"SOA",  "raw",soa,     pa_soa,     0,        cs_soa        ),
+DEEP_TYPE(ptr_raw,"PTR",  "raw",str,     pa_host_raw,0,        cs_domain     ),
+DEEP_TYPE(hinfo,  "HINFO", 0, intstrpair,pa_hinfo,   0,        cs_hinfo      ),
+DEEP_TYPE(mx_raw, "MX",   "raw",intstr,  pa_mx_raw,  di_mx_raw,cs_inthost    ),
+DEEP_TYPE(txt,    "TXT",   0,   manyistr,pa_txt,     0,        cs_txt        ),
+DEEP_TYPE(rp_raw, "RP",   "raw",strpair, pa_rp,      0,        cs_rp         ),
+XTRA_TYPE(srv_raw,"SRV",  "raw",srvraw , pa_srvraw,  di_srv,   cs_srvraw,
+	                                               qdpl_srv, postsort_srv),
+
+FLAT_TYPE(addr,   "A",  "addr", addr,    pa_addr,    di_addr,  cs_addr       ),
+DEEP_TYPE(ns,     "NS", "+addr",hostaddr,pa_hostaddr,di_hostaddr,cs_hostaddr ),
+DEEP_TYPE(ptr,    "PTR","checked",str,   pa_ptr,     0,        cs_domain     ),
+DEEP_TYPE(mx,     "MX", "+addr",inthostaddr,pa_mx,   di_mx,    cs_inthostaddr),
+XTRA_TYPE(srv,    "SRV","+addr",srvha,   pa_srvha,   di_srv,   cs_srvha,
+          	                                       qdpl_srv, postsort_srv),
+
+DEEP_TYPE(soa,    "SOA","822",  soa,     pa_soa,     0,        cs_soa        ),
+DEEP_TYPE(rp,     "RP", "822",  strpair, pa_rp,      0,        cs_rp         ),
 };
 
+static const typeinfo typeinfo_unknown=
+DEEP_TYPE(unknown,0, "unknown",byteblock,pa_opaque,  0,        cs_opaque     );
+
 const typeinfo *adns__findtype(adns_rrtype type) {
   const typeinfo *begin, *end, *mid;
 
+  if (type & adns_r_unknown) return &typeinfo_unknown;
+
   begin= typeinfos;  end= typeinfos+(sizeof(typeinfos)/sizeof(typeinfo));
 
   while (begin < end) {
     mid= begin + ((end-begin)>>1);
-    if (mid->type == type) return mid;
-    if (type > mid->type) begin= mid+1;
+    if (mid->typekey == type) return mid;
+    if (type > mid->typekey) begin= mid+1;
     else end= mid;
   }
   return 0;
